function output = vsearchPlusSetSize()
% Visual Search + Set Size Manipulation
%
% >> vsearchPlus 
% to begin experiment

% think globally, act locally
global fix targ dist gtrialx window white black centerx centery
global diam radi fix_loc oclock1 oclock2 oclock3 oclock4 oclock5
global oclock6 oclock7 oclock8 oclock9 oclock10 oclock11 oclock12
global setsizes dataFile dataFileName formatString deviceN

% only for testing... get this fixed before running
%Screen('Preference', 'SkipSyncTests', 1);

    % Make sure random numbers are random
    rand('seed',sum(100*clock));

    % Load fixation point
    fix = imread('fixation2.jpg');

    % Load stimuli as image arrays
    stim_nN = imread('normalN.tif');
    targ = squeeze(stim_nN(:,:,1));
    stim_bN = imread('backwardN.tif');
    dist = squeeze(stim_bN(:,:,1));

    % Initialize gtobal trial counter
    gtrialx = 0; 

    % Stop character listening
    ListenChar(2);

    % Initialize screen & PTB
    % next line is just for Remote testing functionality
    % and should be commented out when we go live
    %Screen('Preference', 'ConserveVRAM', 64);
    whichScreen = 0;
    window = Screen(whichScreen, 'OpenWindow');
    %window = Screen('OpenWindow', whichScreen, [], [0 0 1400 800]);
    white = WhiteIndex(window); % pixel value for white
    black = BlackIndex(window); % pixel value for black
    HideCursor();

    % Get screen resolution
    res = Screen('Resolution',whichScreen);
    screenw = 1400;%res.width;
    screenh = res.height;
    centerx = screenw/2;
    centery = screenh/2;

    % Define diameter of circular search array
    diam = 250; % in pixels -> which is 5 degrees on Goldstone's iMacs
    radi = diam/2;

    % Calibration rectangles
    rectSize = [0 0 200 200];
    rectNW = CenterRectOnPointd(rectSize, centerx-100, centery-100);
    rectSE = CenterRectOnPointd(rectSize, centerx+100, centery+100);
    % rectNE and rectSW will be painted by jsPsych for calibration

    % Determine locations based on width/height of stimuli
    dims = size(targ);
    stimh = dims(1);
    stimw = dims(2);
    hstimh = stimh/2;
    hstimw = stimw/2;
    % [left, top, right, bottom]
    fix_loc  = [ floor(centerx-8) , floor(centery-8) , floor(centerx+8) , floor(centery+8) ];
    oclock1  = [ floor(centerx+(cosd(60)*radi)-hstimw)  , floor(centery-(sind(60)*radi)-hstimh)  , floor(centerx+(cosd(60)*radi)-hstimw+stimw)  , floor(centery-(sind(60)*radi)-hstimh+stimh)  ]; 
    oclock2  = [ floor(centerx+(cosd(30)*radi)-hstimw)  , floor(centery-(sind(30)*radi)-hstimh)  , floor(centerx+(cosd(30)*radi)-hstimw+stimw)  , floor(centery-(sind(30)*radi)-hstimh+stimh)  ]; 
    oclock3  = [ floor(centerx+(cosd(0)*radi)-hstimw)   , floor(centery-(sind(0)*radi)-hstimh)   , floor(centerx+(cosd(0)*radi)-hstimw+stimw)   , floor(centery-(sind(0)*radi)-hstimh+stimh)   ]; 
    oclock4  = [ floor(centerx+(cosd(330)*radi)-hstimw) , floor(centery-(sind(330)*radi)-hstimh) , floor(centerx+(cosd(330)*radi)-hstimw+stimw) , floor(centery-(sind(330)*radi)-hstimh+stimh) ]; 
    oclock5  = [ floor(centerx+(cosd(300)*radi)-hstimw) , floor(centery-(sind(300)*radi)-hstimh) , floor(centerx+(cosd(300)*radi)-hstimw+stimw) , floor(centery-(sind(300)*radi)-hstimh+stimh) ]; 
    oclock6  = [ floor(centerx+(cosd(270)*radi)-hstimw) , floor(centery-(sind(270)*radi)-hstimh) , floor(centerx+(cosd(270)*radi)-hstimw+stimw) , floor(centery-(sind(270)*radi)-hstimh+stimh) ]; 
    oclock7  = [ floor(centerx+(cosd(240)*radi)-hstimw) , floor(centery-(sind(240)*radi)-hstimh) , floor(centerx+(cosd(240)*radi)-hstimw+stimw) , floor(centery-(sind(240)*radi)-hstimh+stimh) ]; 
    oclock8  = [ floor(centerx+(cosd(210)*radi)-hstimw) , floor(centery-(sind(210)*radi)-hstimh) , floor(centerx+(cosd(210)*radi)-hstimw+stimw) , floor(centery-(sind(210)*radi)-hstimh+stimh) ]; 
    oclock9  = [ floor(centerx+(cosd(180)*radi)-hstimw) , floor(centery-(sind(180)*radi)-hstimh) , floor(centerx+(cosd(180)*radi)-hstimw+stimw) , floor(centery-(sind(180)*radi)-hstimh+stimh) ]; 
    oclock10 = [ floor(centerx+(cosd(150)*radi)-hstimw) , floor(centery-(sind(150)*radi)-hstimh) , floor(centerx+(cosd(150)*radi)-hstimw+stimw) , floor(centery-(sind(150)*radi)-hstimh+stimh) ]; 
    oclock11 = [ floor(centerx+(cosd(120)*radi)-hstimw) , floor(centery-(sind(120)*radi)-hstimh) , floor(centerx+(cosd(120)*radi)-hstimw+stimw) , floor(centery-(sind(120)*radi)-hstimh+stimh) ]; 
    oclock12 = [ floor(centerx+(cosd(90)*radi)-hstimw)  , floor(centery-(sind(90)*radi)-hstimh)  , floor(centerx+(cosd(90)*radi)-hstimw+stimw)  , floor(centery-(sind(90)*radi)-hstimh+stimh)  ]; 

    % Possible set sizes
    setsizes = [2, 3, 4, 6]; % to be randomly assigned per trial

    % Make the output file 
    nonunique = 1; subindn = 1;
    while nonunique
        % See if the file exists
        subinds  = sprintf('%02d',subindn);
        filetry  = strcat('PTB_vsearchData_',subinds,'.csv');
        fileret  = dir(filetry);
        if isempty(fileret)
            nonunique = 0;
        else
            subindn = subindn + 1;
        end
    end
    dataFileName = filetry;
    session = subinds;
    dataFile = fopen(dataFileName, 'a');
    count = fprintf(dataFile, '%s \n', 'session, global_trial_num, trial_type, targetpresence, setsize, targetlocation, response, correctness, responsetime');
    fclose('all');
    formatString = '%s, %d, %s, %d, %d, %d, %d, %d, %1.4f \n';
    
    % get arduino device number
    devices = PsychHID('Devices');
    for i = 1:length(devices)
      if (strcmp(devices(i).manufacturer,'Arduino LLC'))
        deviceN = i;
        fprintf('Detected an Arduino - device number %d\n',i);
      end
    end

    % SCRIPT METHOD
    % ,< - Begin (188 win; 54 mac)
    % 0) - Present a PTB trial (48 win; 39 mac)
    % 1! - Present a jsPsych trial (49 win; 30 mac)
    % 5% - Present a PTB practice trial (53 win; 34 mac)
    % 6^ - Present a jsPsych practice trial (54 win; 35 mac)
    % .> - End (190 win; 55 mac)
    
    % Present calibration patches, then
    % wait for begin code, which is a comma
    %
    % Since KbCheck is a MEX file it takes time to 
    % initialize.  This first KbCheck listens for 
    % a "," to start the experiment.  Future 
    % KbCheck calls will be faster.
    Screen(window, 'FillRect', white);
    Screen(window, 'FillRect', black, rectNW);
    Screen(window, 'FillRect', black, rectSE);
    Screen(window, 'Flip');
    
    % wait for comma to start experiment
    %keysOfInterest=zeros(1,256);
    %keysOfInterest(KbName({',<'}))=1;
    %KbQueueCreate(deviceN, keysOfInterest); 
    
    KbQueueCreate(); % any key will work...
    
    KbQueueWait();
    
    KbQueueRelease();
    %RestrictKeysForKbCheck([188, 54]); % wait for a comma
    %KbWait;
    %while KbCheck; end;

    running = 1;
    Screen(window, 'FillRect', white)
    Screen(window, 'Flip'); 
    %RestrictKeysForKbCheck([]);
    KbQueueCreate(deviceN);

    % Catch keypresses and deal accordingly
    while running
        
        Screen(window, 'FillRect', white)
        Screen(window, 'Flip'); 
        
        %[ dumb1, dumb2, keyCode ] = KbCheck; % determines which key is pressed
        %while KbCheck; end; % allow keys to be released
        
        keysOfInterest = zeros(1,256);
        keysOfInterest([55,30,35,39,34]) = 1;
        KbQueueCreate(deviceN,keysOfInterest);
        KbQueueStart(deviceN); % listen for all keys
        while(1)
            [pressed, firstPress] = KbQueueCheck(deviceN);
            if pressed
                break;
            end

            % Wait for 1 msec to prevent system overload:
            WaitSecs('Yieldsecs', 0.001);
        end
        keyCode = firstPress > 0;
        
        %fprintf('key detected from arduino\n');
        %fprintf('You pressed key %i which is %s\n', find(firstPress), KbName(firstPress));
        %disp(keyCode);
        
        if length(find(keyCode==1)) == 1 % if we didn't get two keys at once
           
            if (find(keyCode)==190 || find(keyCode)==55) % '.' to end experiment
                
                fprintf('ended session\n');
                running = 0;
                
            elseif (find(keyCode)==49 || find(keyCode)==30) % '1' to present jsPsych trial
                
                % Pause for jsPsych trial
                output = jsPsychBlastContainer;
                
                % Save trial data
                % open and close the file every time, to avoid lost data in crashes
                % 'session, global_trial_num, trial_type, targetpresence, setsize, targetlocation, response, correctness, responsetime');
                dataFile = fopen(dataFileName, 'a');
                fprintf(dataFile, formatString, session, gtrialx, 'jsPsych experiment', 99, 99, 99, output(1), 99, output(2));
                fclose(dataFile);
                
            elseif (find(keyCode)==54 || find(keyCode)==35) % '6' to present jPsych practice trial
                
                % Pause for jsPsych trial
                output = jsPsychBlastContainer;
                
                % Save trial data
                % open and close the file every time, to avoid lost data in crashes
                % 'session, global_trial_num, trial_type, targetpresence, setsize, targetlocation, response, correctness, responsetime');
                dataFile = fopen(dataFileName, 'a');
                fprintf(dataFile, formatString, session, gtrialx, 'jsPsych practice', 99, 99, 99, output(1), 99, output(2));
                fclose(dataFile);
                
            elseif (find(keyCode)==48 || find(keyCode)==39) % '0' for PTB trial
                
                % Run an experiment trial
                output = vsearchExper;
                
                % Save trial data
                % open and close the file every time, to avoid lost data in crashes
                % 'session, global_trial_num, trial_type, targetpresence, setsize, targetlocation, response, correctness, responsetime');
                dataFile = fopen(dataFileName, 'a');
                fprintf(dataFile, formatString, session, gtrialx, 'ptb experiment', output(1), output(2), output(3), output(4), output(5), output(6));
                fclose(dataFile);
                
            elseif (find(keyCode)==53 || find(keyCode)==34) % '5' for PTB practice trial
               
                % Run a practice trial
                output = vsearchPract;
                
                % Save trial data
                % open and close the file every time, to avoid lost data in crashes
                % 'session, global_trial_num, trial_type, targetpresence, setsize, targetlocation, response, correctness, responsetime');
                dataFile = fopen(dataFileName, 'a');
                fprintf(dataFile, formatString, session, gtrialx, 'ptb practice', output(1), output(2), output(3), output(4), output(5), output(6));
                fclose(dataFile);
                
            else
                
                running = 1;
                
            end
            
        end
        
    end

    % Close PTB
    Screen('CloseAll');

    % Restart character listening
    WaitSecs(1);
    ListenChar(0);

end % end script

function outArray = jsPsychBlastContainer()

    global gtrialx window white  deviceN
    
    % jsPsych should display the fixation point for 1000ms
    WaitSecs(1); 
    
    % jsPsych should then show the stimulus set
    % and immediately start timing
    Screen(window, 'FillRect', white);
    Screen(window, 'Flip');
   
    % jsPsych should get time (timeout after 2 seconds) and then get response
    KbQueueCreate(deviceN);
    KbQueueStart(deviceN);
    t0 = GetSecs;
    keepgoing = 0;
    while ~keepgoing
        [pressed, firstPress] = KbQueueCheck(deviceN);
        keyCode = firstPress > 0;
        t1 = GetSecs;
        if length(find(keyCode==1)) > 0
            keepgoing = 1;
        end
        if(t1-t0 > 2)
            keepgoing = 1;
        end
    end
    
    % figure out what happened
        if ((t1-t0)<2) % if the trial didn't timeout at 2 seconds
            if length(find(keyCode == 1)) < 2 % if the participant didn't press two keys at once
                if (find(keyCode) == 13) % pressed 'j' for target present
                    response = 1;
                    responsetime = firstPress(13)-t0;
                elseif (find(keyCode) == 9) % pressed 'f' for target absent
                    response = 0;
                    responsetime = firstPress(9)-t0;
                else % something unexpected happened
                    response = -1; 
                    responsetime = -1;
                end 
            else
                response = -1;
                responsetime = -1;
            end
        else 
            response = -1; % there was no keypress due to timeout
            responsetime = -1;
        end
    %responsetime = t1-t0; % response time
    
    outArray = [response responsetime];
    
    % increment global trial counter
    gtrialx = gtrialx + 1;
    
end

function outArray = vsearchPract()

    % think globally, act locally
    global fix targ dist gtrialx window white black centerx centery
    global diam radi fix_loc oclock1 oclock2 oclock3 oclock4 oclock5
    global oclock6 oclock7 oclock8 oclock9 oclock10 oclock11 oclock12
    global setsizes dataFile dataFileName deviceN

    try
        targetPresence = randi(2)-1;
        setsize = setsizes(randi(4));
        targetLocations = setmaker(setsize, targetPresence);
        targetLocation = find(targetLocations == 1);
        if isempty(targetLocation), targetLocation = 0; end
        Screen(window, 'FillRect', white);
        Screen(window, 'PutImage', fix, fix_loc);
        Screen(window, 'Flip'); 
        WaitSecs(1); % just the fixation point for 1000ms
        Screen(window, 'FillRect', white);
        Screen(window, 'PutImage', fix, fix_loc);
        if targetLocations(1)==1  Screen(window, 'PutImage', targ, oclock1) ; elseif targetLocations(1)==-1 Screen(window, 'PutImage', dist, oclock1) ; end
        if targetLocations(2)==1  Screen(window, 'PutImage', targ, oclock2) ; elseif targetLocations(2)==-1 Screen(window, 'PutImage', dist, oclock2) ; end
        if targetLocations(3)==1  Screen(window, 'PutImage', targ, oclock3) ; elseif targetLocations(3)==-1 Screen(window, 'PutImage', dist, oclock3) ; end
        if targetLocations(4)==1  Screen(window, 'PutImage', targ, oclock4) ; elseif targetLocations(4)==-1 Screen(window, 'PutImage', dist, oclock4) ; end
        if targetLocations(5)==1  Screen(window, 'PutImage', targ, oclock5) ; elseif targetLocations(5)==-1 Screen(window, 'PutImage', dist, oclock5) ; end
        if targetLocations(6)==1  Screen(window, 'PutImage', targ, oclock6) ; elseif targetLocations(6)==-1 Screen(window, 'PutImage', dist, oclock6) ; end
        if targetLocations(7)==1  Screen(window, 'PutImage', targ, oclock7) ; elseif targetLocations(7)==-1 Screen(window, 'PutImage', dist, oclock7) ; end
        if targetLocations(8)==1  Screen(window, 'PutImage', targ, oclock8) ; elseif targetLocations(8)==-1 Screen(window, 'PutImage', dist, oclock8) ; end
        if targetLocations(9)==1  Screen(window, 'PutImage', targ, oclock9) ; elseif targetLocations(9)==-1 Screen(window, 'PutImage', dist, oclock9) ; end
        if targetLocations(10)==1  Screen(window, 'PutImage', targ, oclock10) ; elseif targetLocations(10)==-1 Screen(window, 'PutImage', dist, oclock10) ; end
        if targetLocations(11)==1  Screen(window, 'PutImage', targ, oclock11) ; elseif targetLocations(11)==-1 Screen(window, 'PutImage', dist, oclock11) ; end
        if targetLocations(12)==1  Screen(window, 'PutImage', targ, oclock12) ; elseif targetLocations(12)==-1 Screen(window, 'PutImage', dist, oclock12) ; end
        Screen(window, 'Flip');
        
        KbQueueCreate(deviceN);
        KbQueueStart(deviceN);
        t0 = GetSecs;
        keepgoing = 0;
        while ~keepgoing
            [pressed, firstPress] = KbQueueCheck(deviceN);
            keyCode = firstPress > 0;
            t1 = GetSecs;
            if length(find(keyCode==1)) > 0
                keepgoing = 1;
            end
            if(t1-t0 > 2)
                keepgoing = 1;
            end
        end

        % figure out what happened
        if ((t1-t0)<2) % if the trial didn't timeout at 2 seconds
            if length(find(keyCode == 1)) < 2 % if the participant didn't press two keys at once
                if (find(keyCode) == 13) % pressed 'j' for target present
                    response = 1;
                    responsetime = firstPress(13)-t0;
                elseif (find(keyCode) == 9) % pressed 'f' for target absent
                    response = 0;
                    responsetime = firstPress(9)-t0;
                else % something unexpected happened
                    response = -1; 
                    responsetime = -1;
                end 
            else
                response = -1;
                responsetime = -1;
            end
        else 
            response = -1; % there was no keypress due to timeout
            responsetime = -1;
        end
        %responsetime = t1-t0; % response time

        % Test for correctness
        correct = response == targetPresence;

        % create output
        outArray = [targetPresence setsize targetLocation response correct responsetime];
        
        % increment global trial counter
        gtrialx = gtrialx + 1;

        % provide feedback during practice
        righttext  = ['CORRECT'];
        wrongtext  = ['INCORRECT\n'...
                      'press the red button for target absent\n'...
                      'press the white button for target present\n'];
        Screen(window, 'FillRect', white);
        if correct
            [nx, ny, bbox] = DrawFormattedText(window, righttext, 'center', 'center', 0);
            Screen('DrawText', window, '', nx, ny, [255, 0, 0, 255]);
        else
            [nx, ny, bbox] = DrawFormattedText(window, wrongtext, 'center', 'center', 0);
            Screen('DrawText', window, '', nx, ny, [255, 0, 0, 255]);   
        end
        Screen(window, 'Flip');
        WaitSecs(2);
        
    catch
        
        ShowCursor;
        Screen('CloseAll');
        Priority(0);
        psychrethrow(psychlasterror);
        
    end
    
end

function outArray = vsearchExper()

    % think globally, act locally
    global fix targ dist gtrialx window white black centerx centery
    global diam radi fix_loc oclock1 oclock2 oclock3 oclock4 oclock5
    global oclock6 oclock7 oclock8 oclock9 oclock10 oclock11 oclock12
    global setsizes dataFile dataFileName deviceN

    try
        targetPresence = randi(2)-1;
        setsize = setsizes(randi(4));
        targetLocations = setmaker(setsize, targetPresence);
        targetLocation = find(targetLocations == 1);
        if isempty(targetLocation), targetLocation = 0; end
        Screen(window, 'FillRect', white);
        Screen(window, 'PutImage', fix, fix_loc);
        Screen(window, 'Flip'); 
        WaitSecs(1); % just the fixation point for 1000ms
        Screen(window, 'FillRect', white);
        Screen(window, 'PutImage', fix, fix_loc);
        if targetLocations(1)==1  Screen(window, 'PutImage', targ, oclock1) ; elseif targetLocations(1)==-1 Screen(window, 'PutImage', dist, oclock1) ; end
        if targetLocations(2)==1  Screen(window, 'PutImage', targ, oclock2) ; elseif targetLocations(2)==-1 Screen(window, 'PutImage', dist, oclock2) ; end
        if targetLocations(3)==1  Screen(window, 'PutImage', targ, oclock3) ; elseif targetLocations(3)==-1 Screen(window, 'PutImage', dist, oclock3) ; end
        if targetLocations(4)==1  Screen(window, 'PutImage', targ, oclock4) ; elseif targetLocations(4)==-1 Screen(window, 'PutImage', dist, oclock4) ; end
        if targetLocations(5)==1  Screen(window, 'PutImage', targ, oclock5) ; elseif targetLocations(5)==-1 Screen(window, 'PutImage', dist, oclock5) ; end
        if targetLocations(6)==1  Screen(window, 'PutImage', targ, oclock6) ; elseif targetLocations(6)==-1 Screen(window, 'PutImage', dist, oclock6) ; end
        if targetLocations(7)==1  Screen(window, 'PutImage', targ, oclock7) ; elseif targetLocations(7)==-1 Screen(window, 'PutImage', dist, oclock7) ; end
        if targetLocations(8)==1  Screen(window, 'PutImage', targ, oclock8) ; elseif targetLocations(8)==-1 Screen(window, 'PutImage', dist, oclock8) ; end
        if targetLocations(9)==1  Screen(window, 'PutImage', targ, oclock9) ; elseif targetLocations(9)==-1 Screen(window, 'PutImage', dist, oclock9) ; end
        if targetLocations(10)==1  Screen(window, 'PutImage', targ, oclock10) ; elseif targetLocations(10)==-1 Screen(window, 'PutImage', dist, oclock10) ; end
        if targetLocations(11)==1  Screen(window, 'PutImage', targ, oclock11) ; elseif targetLocations(11)==-1 Screen(window, 'PutImage', dist, oclock11) ; end
        if targetLocations(12)==1  Screen(window, 'PutImage', targ, oclock12) ; elseif targetLocations(12)==-1 Screen(window, 'PutImage', dist, oclock12) ; end
        Screen(window, 'Flip');
        
        KbQueueCreate(deviceN);
        KbQueueStart(deviceN);
        t0 = GetSecs;
        keepgoing = 0;
        while ~keepgoing
            [pressed, firstPress] = KbQueueCheck(deviceN);
            keyCode = firstPress > 0;
            t1 = GetSecs;
            if length(find(keyCode==1)) > 0
                keepgoing = 1;
            end
            if(t1-t0 > 2)
                keepgoing = 1;
            end
        end

        % figure out what happened
        if ((t1-t0)<2) % if the trial didn't timeout at 2 seconds
            if length(find(keyCode == 1)) < 2 % if the participant didn't press two keys at once
                if (find(keyCode) == 13) % pressed 'j' for target present
                    response = 1;
                    responsetime = firstPress(13)-t0;
                elseif (find(keyCode) == 9) % pressed 'f' for target absent
                    response = 0;
                    responsetime = firstPress(9)-t0;
                else % something unexpected happened
                    response = -1; 
                    responsetime = -1;
                end 
            else
                response = -1;
                responsetime = -1;
            end
        else 
            response = -1; % there was no keypress due to timeout
            responsetime = -1;
        end
        %responsetime = t1-t0; % response time

        % Test for correctness
        correct = response == targetPresence;
        
        % create output
        outArray = [targetPresence setsize targetLocation response correct responsetime];

        % increment global trial counter
        gtrialx = gtrialx + 1;

    catch
        
        ShowCursor;
        Screen('CloseAll');
        Priority(0);
        psychrethrow(psychlasterror);
        
    end
    
end


% Local function for making the set
% INPUT:  a setsize and targetpresence
% OUTPUT: an array with length 12
%         like [ 0 0 1 0 0 0 0 0 -1 0 0 0 ]
%         where 0 is no item
%               -1 is a distractor
%           and 1 is the target (if present)
function locs = setmaker(setsize, targetPresence)
    locs = zeros(1,12);
    index = randi(12);
    for i = 1:(setsize)
        if targetPresence == 1 && i == 1
            locs(index) = 1;
        else
            locs(index) = -1;
        end
        index = mod(index + (12/setsize),12);
        if index == 0, index = 12; end
    end
end